' =========================================================================
'
'   File...... simple_stepper.spin
'   Purpose... Simple Stepper Motor Demo
'   Author.... translated (with modifications) by Jon Williams
'   E-mail.... jwilliams@efx-tek.com
'   Started...
'   Updated... 17 JUN 2006
'
' =========================================================================
'
' Connections (for L293D DIP):
' --------------------------------------
' P0 -> L293D.1 and L293D.9
' P1 -> L293D.2, L293D.3   -> Phase 1 
' P2 -> L293D.7, L293D.6   -> Phase 2
' P3 -> L293D.10, L293D.11 -> Phase 3
' P4 -> L293D.15, L293D.14 -> Phase 4


CON

  _CLKMODE      = XTAL1 + PLL16X
  _XINFREQ      = 5_000_000

  EN            = 0                                     ' L293D enable pin
  M1            = EN + 1                                ' motor connections
  M4            = M1 + 3                                '  (contiguous)
  
  STEPS_PER_REV = 48                                    ' steps per revolution


VAR

  long  idx, stpIdx, stpDelay


PUB main

  dira[M4..EN]~~                                        ' make pins outputs
  outa[EN]~~                                            ' enable the L293D

  repeat
    stpDelay := 5                                       ' set step delay 
    repeat idx from 1 to STEPS_PER_REV                  ' one rev forward
      stepFwd
      
    pause(500)

    repeat while (stpDelay < 50)                        ' modified speed loop
      repeat idx from 1 to STEPS_PER_REV                ' one revolution
        stepRev                                         '   in reverse
      stpDelay := stpDelay * 125 / 100                  ' add 25% to step delay

    pause(500)  


PRI stepFwd

  stpIdx := ++stpIdx // 4                               ' point to next step
  outa[M4..M1] := Steps[stpIdx]                         ' update outputs
  pause(stpDelay)


PRI stepRev

  stpIdx := (stpIdx + 3) // 4                           ' point to previous step
  outa[M4..M1] := Steps[stpIdx]                         ' update outputs
  pause(stpDelay)
  

PRI pause(ms) | c

  c := cnt                                              ' sync with system counter
  repeat until (ms-- == 0)                              ' repeat while time left
    waitcnt(c += clkfreq / 1000)                        ' wait 1 ms
   

DAT

  Steps       byte      %0011, %0110, %1100, %1001      ' step table

  